// Com.cpp: implementation of the CCom class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "Com.h"
#include <iostream.h>

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CCom::CCom()
{
	 m_hCom = 0;                //HANDLER del port serie.
	 m_dwBaudRate = CBR_9600;   //Velocitat de transmissi(9600 Bauds)
	 m_lpszPortNum = "com1";    //String amb el nom del port.
	 m_byParity = NOPARITY;	    //Determina transmissi amb paritat o sense.
	 m_byStopBits = ONESTOPBIT; //Nmero de bits de parada.
	 m_byByteSize = 8;          //Longitud del byte de dades.

}
CCom::~CCom()
{
	CCom::Close();
	CloseHandle(m_ov.hEvent);

}

BOOL CCom::Open(){
	
	DCB dcb;        //Data comunication block.Guarda els parmetres de configuraci del
	                //dispositiu de comunicacions.
    /////////////////////////////////////////////////////////////////////////////////
	//Creem el fitxer: m_hCom=CreateFile(,,,,..,);
	//--> m_lpszPortNum: Nom del port al que volem LLegir/Escriure.
	//--> GENERIC_READ | GENERIC_WRITE: Donem accs de lectura i d'escriptura.
	//--> 0 :...........................No compartim el fitxer.
	//--> NULL:.........................No hi ha seguritat.
	//--> OPEN_EXISTING:................Es necessari per el obrir un Port COM.
	//--> FILE_ATTRIBUTE_NORMAL:........El fitxer no disposa d'altres atributs.
	//--> NULL:
	//////////////////////////////////////////////////////////////////////////////////
	//////////////////////////////////////////////////////////////////////////////////
	m_hCom = CreateFile( m_lpszPortNum,             
						 GENERIC_READ|GENERIC_WRITE,
						 0,							
						 NULL,					
						 OPEN_EXISTING,
						 FILE_ATTRIBUTE_NORMAL,
						 NULL);
	//////////////////////////////////////////////////////////////////////////////////
	//////////////////////////////////////////////////////////////////////////////////
	// CreateFile retorna un HANDLE al fitxer especificat si aquest, s'ha obert 
	// correctament. Si no s'ha obert correctament retorna INVALID_HANDLE_VALUE i
	// es pot demanar ms informaci de l'error cridant la funci GetLastError.
	//////////////////////////////////////////////////////////////////////////////////
	//////////////////////////////////////////////////////////////////////////////////
	// Comprovem si el HANDLE ha estat retornat correctament. Si no ha estat retornat
	// correctament, llavors sortim de la funci retornant FALSE per indicar que hi ha 
	// agut un error.
	///////////////////////////////////////////////////////////////////////////////////
	///////////////////////////////////////////////////////////////////////////////////
	if( m_hCom == INVALID_HANDLE_VALUE){
		return FALSE;
	}
	///////////////////////////////////////////////////////////////////////////////////
	///////////////////////////////////////////////////////////////////////////////////
	// GetCommState recupera la configuraci que controla l'actual dispositiu de
	// comunicacions on apunta el HANDLER que se li passa per parmetre. Si la informaci 
	// no es recupera correctament, tanquem e fitxer i retornem FALSE per indicar que hi 
	// ha hagut un error.
    ///////////////////////////////////////////////////////////////////////////////////
	if(!GetCommState(m_hCom,&dcb)){
		CCom::Close();
		return FALSE;
	}
	////////////////////////////////////////////////////////////////////////////////////
	////////////////////////////////////////////////////////////////////////////////////
	// Modifiquem els elements de l'estructura corresponents a la velocitat de transmisi
	// la longitud del Byte de dades, la paritat i el nmero de bits de stop.
	////////////////////////////////////////////////////////////////////////////////////
	////////////////////////////////////////////////////////////////////////////////////
	unsigned char character='B';
	dcb.BaudRate = m_dwBaudRate;
	dcb.ByteSize = m_byByteSize;
	dcb.Parity   = m_byParity;
	dcb.StopBits = m_byStopBits;
	dcb.EvtChar = character;            //Carcter utilitzat per indica el final de la trama.

	////////////////////////////////////////////////////////////////////////////////////
	////////////////////////////////////////////////////////////////////////////////////
	// Inicialitzem el port amb els valors que cont la estructura DCB. Si al inicialitzar
	// hi ha hagut algn error, retornem FALSE per indicar-lo.
	////////////////////////////////////////////////////////////////////////////////////
	if(!SetCommState(m_hCom, &dcb)){
		CCom::Close();
		return FALSE;
	}
	////////////////////////////////////////////////////////////////////////////////////
	// Si hem arribat en aquest punt, vol dir que hem realitzat la inicialitzaci del 
	// port de maner correte.Aix doncs, retornem TRUE.
	/////////////////////////////////////////////////////////////////////////////////////
	return TRUE;
}

DWORD CCom::WriteData(unsigned char *pdata,DWORD len){


	BOOL bSuccess;
	DWORD written = 0;
	/////////////////////////////////////////////////////////////////////////////////////
	/////////////////////////////////////////////////////////////////////////////////////
	// Si la longitud que li hem passat perque escrigui, s menor que 1, no te sentit.
	// Retornem 0 per indicar que no s'ha escrit cap byte.
	/////////////////////////////////////////////////////////////////////////////////////
	/////////////////////////////////////////////////////////////////////////////////////
	if(len < 1){
		return 0;
	}
	/////////////////////////////////////////////////////////////////////////////////////
	/////////////////////////////////////////////////////////////////////////////////////
	// WriteFile escriu dades en el fitxer on apunta el HANDLER. Un cop ha acabat, modifica
	// el parmetre written que indica el nmero de bytes escrits.
	// 
	// m_hCom:  HANDLER al fitxer on escribim.
	// pdata:	Punter a l'array de dades d'on llegim.
	// len:     Nmero de Bytes a escriure.
	// &written:Punter a la variable on es guarda el nmero de bytes llegits correctament.
	// NULL:    El parmetre ovellaped ha d'estar a NULL per fora.
	//
	// Si el fitxer s'ha escrit correctament i el nmero de bytes escrits es igual al n
	// de bytes llegits, l'operaci s'ha realitzat amb xit i retornem el nmero de bytes 
	// escrits.
	/////////////////////////////////////////////////////////////////////////////////////
	/////////////////////////////////////////////////////////////////////////////////////
	bSuccess = WriteFile(m_hCom,pdata,len,&written,NULL);   
	if((!bSuccess) || (len!= written)){
		if(bSuccess == 0){
			return(0);
		}
	}
	return (written);
}

DWORD CCom::ReadData(unsigned char *pdest,DWORD dwMaxWait){
	

	DWORD result =0;
	DWORD read = 0; //Nmero de bytes llegits.
	DWORD mask =0;	//Variable de 32 bits que indica l'event que
					//ha ocurregut.
	COMSTAT COMstatus;
	DWORD ErrCode;


	int fOverlapped=FALSE;

	//////////////////////////////////////////////////////////////////////////////////////////////
	//////////////////////////////////////////////////////////////////////////////////////////////
	// Inicialitzem un objecte d'event.
	//////////////////////////////////////////////////////////////////////////////////////////////
	if(!(m_ov.hEvent=CreateEvent(NULL,FALSE,FALSE,NULL))){
		return FALSE;
	}
	
	//////////////////////////////////////////////////////////////////////////////////////////////
	//////////////////////////////////////////////////////////////////////////////////////////////
	// La funci SetCommMask, especifica un conjunt d'events a ser monitoritzats per un dispositiu
	// de comunicaci. A la funic li passem un HANDLER al dispositiu i la mascar d'events.
	// Si la funci no crea correctament la mascara, retornem FALSE per indicar que hi ha hagut un 
	// error. En el nostre cas li passem la mascara corresponent al Event que indica que ha arribat
	// un carcter en el buffer d'entrada del dispositiu de comunicacions.  
	///////////////////////////////////////////////////////////////////////////////////////////////
	if(!SetCommMask(m_hCom, EV_RXFLAG)){
		return FALSE;
	}
	///////////////////////////////////////////////////////////////////////////////////////////////
	///////////////////////////////////////////////////////////////////////////////////////////////
	// Establim el time out.
	///////////////////////////////////////////////////////////////////////////////////////////////
	///////////////////////////////////////////////////////////////////////////////////////////////
	Set_Read_Time_Out(dwMaxWait);
	///////////////////////////////////////////////////////////////////////////////////////////////
	///////////////////////////////////////////////////////////////////////////////////////////////
	// Amb WaitCommEvent, ens esperem fins que succeix algn dels events de la mascara.
	///////////////////////////////////////////////////////////////////////////////////////////////
	///////////////////////////////////////////////////////////////////////////////////////////////
	if(!WaitCommEvent(m_hCom, &mask, NULL)){
		return FALSE;
	}
	///////////////////////////////////////////////////////////////////////////////////////////////
	///////////////////////////////////////////////////////////////////////////////////////////////
	// Si som aqu es que ha arribat algun carcter al buffer.
	///////////////////////////////////////////////////////////////////////////////////////////////
	///////////////////////////////////////////////////////////////////////////////////////////////
	ClearCommError(m_hCom,&ErrCode,&COMstatus);
				
	if(ReadFile(m_hCom,	pdest,COMstatus.cbInQue,&read,NULL)){
		//return COMstatus.cbInQue
		return read;
	}
	else return FALSE;
}

DWORD CCom::ReadData(unsigned char *pdest){
	
	
	DWORD result =0;
	DWORD read = 0; //Nmero de bytes llegits.
	DWORD mask =0;	//Variable de 32 bits que indica l'event que
					//ha ocurregut.
	COMSTAT COMstatus; //Aquesta variable pot guardar informaci del port. En el nostre cas ens diu el 
					   //Nmero de bytes que hi ha al buffer.
	DWORD ErrCode;


	//////////////////////////////////////////////////////////////////////////////////////////////
	//////////////////////////////////////////////////////////////////////////////////////////////
	// Inicialitzem un objecte d'event.
	//////////////////////////////////////////////////////////////////////////////////////////////
	if(!(m_ov.hEvent=CreateEvent(NULL,FALSE,FALSE,NULL))){
		return FALSE;
	}
	
	//////////////////////////////////////////////////////////////////////////////////////////////
	//////////////////////////////////////////////////////////////////////////////////////////////
	// La funci SetCommMask, especifica un conjunt d'events a ser monitoritzats per un dispositiu
	// de comunicaci. A la funic li passem un HANDLER al dispositiu i la mascar d'events.
	// Si la funci no crea correctament la mascara, retornem FALSE per indicar que hi ha hagut un 
	// error. En el nostre cas li passem la mascara corresponent al Event que indica que ha arribat
	// un carcter en el buffer d'entrada del dispositiu de comunicacions.  
	///////////////////////////////////////////////////////////////////////////////////////////////
	if(!SetCommMask(m_hCom, EV_RXFLAG)){
		return FALSE;
	}
	///////////////////////////////////////////////////////////////
	///////////////////////////////////////////////////////////////////////////////////////////////
	// Amb WaitCommEvent, ens esperem fins que succeix algn dels events de la mascara.
	///////////////////////////////////////////////////////////////////////////////////////////////
	///////////////////////////////////////////////////////////////////////////////////////////////
	if(!WaitCommEvent(m_hCom, &mask, NULL)){
		return FALSE;
	}
	///////////////////////////////////////////////////////////////////////////////////////////////
	///////////////////////////////////////////////////////////////////////////////////////////////
	// Si som aqu es que ha arribat el carcter d'event al al buffer.
	///////////////////////////////////////////////////////////////////////////////////////////////
	///////////////////////////////////////////////////////////////////////////////////////////////
	ClearCommError(m_hCom,&ErrCode,&COMstatus);
				
	if(ReadFile(m_hCom,	pdest,COMstatus.cbInQue,&read,NULL)){
		//return COMstatus.cbInQue
		return read;
	}
	else return FALSE;
}

void CCom::Set_Read_Time_Out(DWORD ReadTimeOut){

	COMMTIMEOUTS timeout;
	GetCommTimeouts(m_hCom,&timeout);
	
	//Posem a zero el multiplicador.
	timeout.ReadTotalTimeoutMultiplier = 0;
	timeout.WriteTotalTimeoutConstant = 0;
	timeout.ReadIntervalTimeout = ReadTimeOut;
	SetCommTimeouts(m_hCom,&timeout);
}

void CCom::Close(){
	if( m_hCom >0 ){
		CloseHandle(m_hCom);
	}
	m_hCom = 0;
}

int CCom::LeerTrama(unsigned char *TrRec){

	unsigned char tmp[256];
	int  num1;
	int i=0;

	//Netegem el buffer d'entrada.
	PurgeComm(m_hCom,PURGE_RXCLEAR);
	//////////////////////////////////////////////////////////////////////////////////////////
	//////////////////////////////////////////////////////////////////////////////////////////
	// LLegim el buffer i ens quedem amb el nmero de bytes llegits.En principi, la funci 
	// ReadData espera a llegir fins que ha arribat el carcter 0xFF que es el de final de 
	// dada.
	//////////////////////////////////////////////////////////////////////////////////////////
	//////////////////////////////////////////////////////////////////////////////////////////
	num1=ReadData(tmp);
	//////////////////////////////////////////////////////////////////////////////////////////
	//////////////////////////////////////////////////////////////////////////////////////////
	// num1 cont el nmero de bytes llegits. Passem el contingut dels bytes llegits al buffer
	// TrRec.
	//////////////////////////////////////////////////////////////////////////////////////////
	//////////////////////////////////////////////////////////////////////////////////////////
	for(int k=0;k<num1;k++)
			TrRec[k]=tmp[k];
	//////////////////////////////////////////////////////////////////////////////////////////
	//////////////////////////////////////////////////////////////////////////////////////////
	// Retornem el nmero de bytes llegits.
	//////////////////////////////////////////////////////////////////////////////////////////
	//////////////////////////////////////////////////////////////////////////////////////////
	return num1;
}

DWORD CCom::Envio(/*TTrama Tr*/unsigned char *Tr){
	int i=0;
	do{
	}while(Tr[i++]!=0xff);
	return WriteData(Tr,i);
}








